<?php

declare (strict_types=1);
namespace MailerLite_Woo\Http\Client\Common\HttpClientPool;

use MailerLite_Woo\Http\Client\Common\Exception\HttpClientNotFoundException;
use MailerLite_Woo\Http\Client\Common\HttpClientPool as HttpClientPoolInterface;
use MailerLite_Woo\Http\Client\HttpAsyncClient;
use MailerLite_Woo\Psr\Http\Client\ClientInterface;
use MailerLite_Woo\Psr\Http\Message\RequestInterface;
use MailerLite_Woo\Psr\Http\Message\ResponseInterface;
/**
 * A http client pool allows to send requests on a pool of different http client using a specific strategy (least used,
 * round robin, ...).
 */
abstract class HttpClientPool implements \MailerLite_Woo\Http\Client\Common\HttpClientPool
{
    /**
     * @var HttpClientPoolItem[]
     */
    protected $clientPool = [];
    /**
     * Add a client to the pool.
     *
     * @param ClientInterface|HttpAsyncClient $client
     */
    public function addHttpClient($client) : void
    {
        // no need to check for HttpClientPoolItem here, since it extends the other interfaces
        if (!$client instanceof \MailerLite_Woo\Psr\Http\Client\ClientInterface && !$client instanceof \MailerLite_Woo\Http\Client\HttpAsyncClient) {
            throw new \TypeError(\sprintf('%s::addHttpClient(): Argument #1 ($client) must be of type %s|%s, %s given', self::class, \MailerLite_Woo\Psr\Http\Client\ClientInterface::class, \MailerLite_Woo\Http\Client\HttpAsyncClient::class, \get_debug_type($client)));
        }
        if (!$client instanceof \MailerLite_Woo\Http\Client\Common\HttpClientPool\HttpClientPoolItem) {
            $client = new \MailerLite_Woo\Http\Client\Common\HttpClientPool\HttpClientPoolItem($client);
        }
        $this->clientPool[] = $client;
    }
    /**
     * Return an http client given a specific strategy.
     *
     * @throws HttpClientNotFoundException When no http client has been found into the pool
     *
     * @return HttpClientPoolItem Return a http client that can do both sync or async
     */
    protected abstract function chooseHttpClient() : \MailerLite_Woo\Http\Client\Common\HttpClientPool\HttpClientPoolItem;
    /**
     * {@inheritdoc}
     */
    public function sendAsyncRequest(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request)
    {
        return $this->chooseHttpClient()->sendAsyncRequest($request);
    }
    /**
     * {@inheritdoc}
     */
    public function sendRequest(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request) : \MailerLite_Woo\Psr\Http\Message\ResponseInterface
    {
        return $this->chooseHttpClient()->sendRequest($request);
    }
}
