<?php

declare (strict_types=1);
namespace MailerLite_Woo\Http\Client\Common;

use MailerLite_Woo\Http\Client\Common\Exception\HttpClientNoMatchException;
use MailerLite_Woo\Http\Client\HttpAsyncClient;
use MailerLite_Woo\Http\Message\RequestMatcher;
use MailerLite_Woo\Psr\Http\Client\ClientInterface;
use MailerLite_Woo\Psr\Http\Message\RequestInterface;
use MailerLite_Woo\Psr\Http\Message\ResponseInterface;
/**
 * {@inheritdoc}
 *
 * @author Joel Wurtz <joel.wurtz@gmail.com>
 */
final class HttpClientRouter implements \MailerLite_Woo\Http\Client\Common\HttpClientRouterInterface
{
    /**
     * @var (array{matcher: RequestMatcher, client: FlexibleHttpClient})[]
     */
    private $clients = [];
    /**
     * {@inheritdoc}
     */
    public function sendRequest(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request) : \MailerLite_Woo\Psr\Http\Message\ResponseInterface
    {
        return $this->chooseHttpClient($request)->sendRequest($request);
    }
    /**
     * {@inheritdoc}
     */
    public function sendAsyncRequest(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request)
    {
        return $this->chooseHttpClient($request)->sendAsyncRequest($request);
    }
    /**
     * Add a client to the router.
     *
     * @param ClientInterface|HttpAsyncClient $client
     */
    public function addClient($client, \MailerLite_Woo\Http\Message\RequestMatcher $requestMatcher) : void
    {
        if (!$client instanceof \MailerLite_Woo\Psr\Http\Client\ClientInterface && !$client instanceof \MailerLite_Woo\Http\Client\HttpAsyncClient) {
            throw new \TypeError(\sprintf('%s::addClient(): Argument #1 ($client) must be of type %s|%s, %s given', self::class, \MailerLite_Woo\Psr\Http\Client\ClientInterface::class, \MailerLite_Woo\Http\Client\HttpAsyncClient::class, \get_debug_type($client)));
        }
        $this->clients[] = ['matcher' => $requestMatcher, 'client' => new \MailerLite_Woo\Http\Client\Common\FlexibleHttpClient($client)];
    }
    /**
     * Choose an HTTP client given a specific request.
     */
    private function chooseHttpClient(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request) : \MailerLite_Woo\Http\Client\Common\FlexibleHttpClient
    {
        foreach ($this->clients as $client) {
            if ($client['matcher']->matches($request)) {
                return $client['client'];
            }
        }
        throw new \MailerLite_Woo\Http\Client\Common\Exception\HttpClientNoMatchException('No client found for the specified request', $request);
    }
}
