<?php

declare (strict_types=1);
namespace MailerLite_Woo\Http\Client\Common\Plugin;

use MailerLite_Woo\Http\Client\Common\Plugin;
use MailerLite_Woo\Http\Promise\Promise;
use MailerLite_Woo\Psr\Http\Message\RequestInterface;
/**
 * Append headers to the request.
 *
 * If the header already exists the value will be appended to the current value.
 *
 * This only makes sense for headers that can have multiple values like 'Forwarded'
 *
 * @see https://en.wikipedia.org/wiki/List_of_HTTP_header_fields
 *
 * @author Soufiane Ghzal <sghzal@gmail.com>
 */
final class HeaderAppendPlugin implements \MailerLite_Woo\Http\Client\Common\Plugin
{
    /**
     * @var array
     */
    private $headers;
    /**
     * @param array $headers Hashmap of header name to header value
     */
    public function __construct(array $headers)
    {
        $this->headers = $headers;
    }
    /**
     * {@inheritdoc}
     */
    public function handleRequest(\MailerLite_Woo\Psr\Http\Message\RequestInterface $request, callable $next, callable $first) : \MailerLite_Woo\Http\Promise\Promise
    {
        foreach ($this->headers as $header => $headerValue) {
            $request = $request->withAddedHeader($header, $headerValue);
        }
        return $next($request);
    }
}
